@@ -93,7 +93,7 @@ check.earth.matches.lm <- function(earth, lm, newdata=data[c(3,1,9),],
     if(is.null(earth$weights))
         stopifnot(almost.equal(earth$rss.per.response, earth$rss, max=1e-10))
 }
-# check that earth model matches lm model in all essential details
+# check that earth-glm model matches glm model in all essential details
 check.earth.matches.glm <- function(earth, glm, newdata=data[c(3,1,9),],
                                    check.coef.names=TRUE,
                                    check.casenames=FALSE,
@@ -203,7 +203,7 @@ print.devratio <- function(s, mod)
 # linear model
 
 n <- 100
-set.seed(2018)
+set.seed(2019)
 x1 <- ((1:n) + runif(n, min=0, max=10)) / n
 set.seed(2019)
 x2 <- ((1:n) + runif(n, min=0, max=10)) / n
@@ -231,17 +231,17 @@ cat("==summary(earth4, details=TRUE)==\n")
 print(summary(earth4, details=TRUE))
 
 par(mfrow=c(4, 2), mar=c(3, 3, 3, 1), mgp=c(1.5, 0.5, 0), oma=c(0, 0, 5, 0))
-set.seed(2018)
+set.seed(2019)
 plotmo(lm4, trace=0, pt.col=2, do.par=FALSE)
 mtext(
     "row1: lm4\nrow2: earth4\nrow3: lm4   grid.levels=list(myoffset=-3)\nrow4: earth4   grid.levels=list(myoffset=-3)",
     outer=TRUE, cex=.8)
-set.seed(2018)
+set.seed(2019)
 plotmo(earth4, trace=0, pt.col=2, do.par=FALSE)
 empty.plot()
-set.seed(2018)
+set.seed(2019)
 plotmo(lm4, trace=0, pt.col=2, do.par=FALSE, grid.levels=list(myoffset=-3))
-set.seed(2018)
+set.seed(2019)
 plotmo(earth4, trace=0, pt.col=2, do.par=FALSE, grid.levels=list(myoffset=-3))
 par(old.par)
 
@@ -281,6 +281,19 @@ pois <- glm(Claims ~ Group + Age + day + offset(log(Holders)),
 earth.pois.linpreds <- earth(Claims ~ offset(log(Holders)) + Group + Age + day,
                              data = Ins, glm=list(family = poisson),
                              linpreds=TRUE, thresh=0, penalty=-1)
+
+stopifnot(isTRUE(all.equal(coef(earth.pois.linpreds), coefficients(earth.pois.linpreds))))
+stopifnot(isTRUE(all.equal(coef(earth.pois.linpreds, type="glm"), coefficients(earth.pois.linpreds, type="glm"))))
+stopifnot(isTRUE(all.equal(coef(earth.pois.linpreds, type="earth"), coefficients(earth.pois.linpreds, type="earth"))))
+stopifnot(identical(names(coef(earth.pois.linpreds)), rownames(earth.pois.linpreds$coefficients)))
+stopifnot(identical(names(coef(earth.pois.linpreds)), rownames(earth.pois.linpreds$glm.coefficients)))
+stopifnot(identical(names(coef(earth.pois.linpreds, type="glm")), rownames(earth.pois.linpreds$glm.coefficients)))
+stopifnot(max(abs(coef(earth.pois.linpreds) - earth.pois.linpreds$glm.coefficients)) == 0)
+stopifnot(max(abs(coef(earth.pois.linpreds, type="response") - earth.pois.linpreds$glm.coefficients)) == 0)
+stopifnot(max(abs(coef(earth.pois.linpreds, type="earth") - earth.pois.linpreds$coefficients)) == 0)
+stopifnot(max(abs(coef(earth.pois.linpreds) - earth.pois.linpreds$glm.list[[1]]$coefficients)) == 0)
+stopifnot(max(abs(coef(earth.pois.linpreds, type="glm") - earth.pois.linpreds$coefficients)) > 99)
+
 check.earth.matches.glm(earth.pois.linpreds, pois, newdata=Ins[4:6,])
 earth.pois <- earth(Claims ~ Group + Age  + day + offset(log(Holders)),
                     data = Ins, glm=list(family = poisson))
@@ -299,16 +312,16 @@ print.devratio("earth.pois",            earth.pois$glm.list[[1]])
 print.devratio("earth.pois.no.penalty", earth.pois.no.penalty$glm.list[[1]])
 
 par(mfrow=c(3, 4), mar=c(3, 3, 3, 1), mgp=c(1.5, 0.5, 0), oma=c(0, 0, 5, 0))
-set.seed(2018)
+set.seed(2019)
 plotmo(pois, trace=0, pt.col=2, do.par=FALSE, ylim=c(0,50))
 mtext(sprint(
     "row1: pois (%s)\nrow2: earth.pois.linpreds (%s)\nrow3: earth.pois.linpreds(all1=TRUE)",
     devratio(pois), devratio(earth.pois.linpreds)),
     outer=TRUE, cex=.8)
-set.seed(2018)
+set.seed(2019)
 plotmo(earth.pois.linpreds, trace=0, pt.col=2, do.par=FALSE, ylim=c(0,50))
 empty.plot()
-set.seed(2018)
+set.seed(2019)
 plotmo(earth.pois.linpreds, all1=T, trace=-1, pt.col=2, do.par=FALSE, ylim=c(0,50))
 par(old.par)
 
@@ -316,15 +329,15 @@ plotres(pois, type="response", caption='pois, type="response"')
 plotres(earth.pois.linpreds, type="response", caption='earth.pois.linpreds, type="response"')
 
 par(mfrow=c(3, 4), mar=c(3, 3, 3, 1), mgp=c(1.5, 0.5, 0), oma=c(0, 0, 5, 0))
-set.seed(2018)
+set.seed(2019)
 plotmo(pois, trace=0, pt.col=2, do.par=FALSE, ylim=c(0,50), grid.levels=list(Holders=20))
 mtext(
     "----- grid.levels=list(Holders=20)) -----\nrow1: pois\nrow2: earth.pois.linpreds\nrow3: earth.pois.linpreds(all1=TRUE)",
     outer=TRUE, cex=.8)
-set.seed(2018)
+set.seed(2019)
 plotmo(earth.pois.linpreds, trace=0, pt.col=2, do.par=FALSE, ylim=c(0,50), grid.levels=list(Holders=20))
 empty.plot()
-set.seed(2018)
+set.seed(2019)
 plotmo(earth.pois.linpreds, all1=T, trace=-1, pt.col=2, do.par=FALSE, ylim=c(0,50), grid.levels=list(Holders=20))
 par(old.par)
 
@@ -386,7 +399,7 @@ plotmo(earth.pois.linpreds.weights.some.zero, caption="earth.pois.linpreds.weigh
 
 data(trees)
 tr <- trees
-set.seed(2018)
+set.seed(2019)
 tr$Vol2 <- tr$Volume + 10 * rnorm(nrow(tr))
 
 earth10 <- earth(Volume ~ Girth + offset(log(Height)), data=tr,
@@ -443,7 +456,7 @@ plotmo(earth.pois.multiple.response, nresponse=1, pt.col=2)
 
 data(trees)
 tr <- trees
-set.seed(2018)
+set.seed(2019)
 tr$Vol2 <- tr$Volume + 10 * rnorm(nrow(tr))
 my.weights <- 1:nrow(tr)
 my.weights[3] <- 0
